/*  jis2gb (JIS Kanji to GuoBiao Hanzi converter) Version 1.4
		by Koichi Yasuoka, May 11, 1995 */

#include <stdio.h>
#ifndef UNDEFCHAR
#define UNDEFCHAR 0x2020
#endif
extern unsigned short *JX0208GB2312[],*JX0208GB8565[];
extern unsigned short *JX0212GB2312[],*JX0212GB8565[];
FILE *fp;
int GB,SJIS,KANJI,HANZI,ISATTY;

main(argc,argv)
int argc;
char *argv[];
{
  int i;
  GB=SJIS=KANJI=HANZI=0;
  ISATTY=isatty(1);
  while(argc>1){
    if(argv[1][0]!='-')
      break;
    for(i=1;argv[1][i]>0;i++){
      switch(argv[1][i]){
	case 's':
	  SJIS=1;
	  break;
	case 'g':
	  GB|=1;
	  break;
	case 'b':
	  if(GB==5){
	    fprintf(stderr,"jis2gb: option -h contradicts -b\n");
	    exit(2);
	  }
	  GB|=2;
	  break;
	case 'h':
	  if(GB&2){
	    fprintf(stderr,"jis2gb: option -b contradicts -h\n");
	    exit(2);
	  }
	  GB=5;
	  break;
	case 'i':
	  ISATTY=1;
	  break;
	default:
	  fprintf(stderr,"Usage: jis2gb [-sgbhi] [file...]\n");
	  exit(2);
      }
    }
    argc--;
    argv++;
  }
  if(argc==1){
    fp=stdin;
    jis2gb();
    exit(0);
  }
  while(argc>1){
    if((fp=fopen(argv[1],"r"))==NULL)
      fprintf(stderr,"Cannot open %s\n",argv[1]);
    else{
      jis2gb();
      fclose(fp);
    }
    argc--;
    argv++;
  }
  exit(0);
}

jis2gb(){
  int h,l;
  while((l=kanji())>=0){
    if(l<256)
      hanzi0(l);
    else if(l<65536){
      h=(l>>8)-161;
      l=(l&255)-161;
      if((GB&1)==0&&JX0208GB8565[h][l]>0)
	hanzi1(JX0208GB8565[h][l]);
      else if(JX0208GB2312[h][l]>0)
	hanzi1(JX0208GB2312[h][l]);
      else
	hanzi1(UNDEFCHAR);
    }
    else{
      h=(l>>8)-417;
      l=(l&255)-161;
      if((GB&1)==0&&JX0212GB8565[h][l]>0)
	hanzi1(JX0212GB8565[h][l]);
      else if(JX0212GB2312[h][l]>0)
	hanzi1(JX0212GB2312[h][l]);
      else
	hanzi1(UNDEFCHAR);
    }
    if(ISATTY)
      fflush(stdout);
  }
}

int kanatbl[]={
         0xa1a3,0xa1d6,0xa1d7,0xa1a2,0xa1a6,0xa5f2,0xa5a1,
  0xa5a3,0xa5a5,0xa5a7,0xa5a9,0xa5e3,0xa5e5,0xa5e7,0xa5c3,
  0xa1bc,0xa5a2,0xa5a4,0xa5a6,0xa5a8,0xa5aa,0xa5ab,0xa5ad,
  0xa5af,0xa5b1,0xa5b3,0xa5b5,0xa5b7,0xa5b9,0xa5bb,0xa5bd,
  0xa5bf,0xa5c1,0xa5c4,0xa5c6,0xa5c8,0xa5ca,0xa5cb,0xa5cc,
  0xa5cd,0xa5ce,0xa5cf,0xa5d2,0xa5d5,0xa5d8,0xa5db,0xa5de,
  0xa5df,0xa5e0,0xa5e1,0xa5e2,0xa5e4,0xa5e6,0xa5e8,0xa5e9,
  0xa5ea,0xa5eb,0xa5ec,0xa5ed,0xa5ef,0xa5f3,0xa1ab,0xa1ac
};

kanji(){
  int i,j;
  while((i=fgetc(fp))>=0){
    if(i==27){
      if((i=fgetc(fp))=='$'){
	if((i=fgetc(fp))=='B'){
	  KANJI=2;
	  continue;
	}
	else if(i=='@'){
	  KANJI=2;
	  continue;
	}
	else if(i=='('){
	  if((j=fgetc(fp))=='D'){
	    KANJI=4;
	    continue;
	  }
	  else if(j=='B'||j=='@'){
	    KANJI=2;
	    continue;
	  }
	  ungetc(j,fp);
	}
	KANJI=0;
	ungetc(i,fp);
	ungetc('$',fp);
      }
      else if(i=='('){
	if((i=fgetc(fp))=='I'){
	  KANJI=6;
	  continue;
	}
	else if(KANJI>1&&(i=='B'||i=='J')){
	  KANJI=0;
	  continue;
	}
	KANJI=0;
	ungetc(i,fp);
	ungetc('(',fp);
      }
      else if(i=='&'){
	if((j=fgetc(fp))=='@')
	  continue;
	ungetc(j,fp);
	ungetc('&',fp);
      }
      else
	ungetc(i,fp);
      KANJI|=1;
      return(27);
    }
    else if(SJIS==0&&i==142){
      j=fgetc(fp);
      if((j>32&&j<96)||(j>160&&j<224))
	return(kanatbl[(j&127)-33]);
      ungetc(j,fp);
    }
    else if(SJIS==0&&i>160&&i<255){
      j=fgetc(fp);
      if(j>160&&j<255)
	return((i<<8)+j);
      ungetc(j,fp);
    }
    else if(SJIS==1&&i>128&&i<240&&i!=160){
      if(i>160&&i<224)
	return(kanatbl[i-161]);
      if(i>223)
	i-=64;
      j=fgetc(fp);
      if(j>63&&j<127)
	return((i<<9)+j-24735);
      else if(j>127&&j<159)
	return((i<<9)+j-24736);
      else if(j>158&&j<253)
	return((i<<9)+j-24574);
      ungetc(j,fp);
      if(i>159)
	i+=64;
    }
    else if(KANJI==2&&i>32&&i<127){
      j=fgetc(fp);
      if(j>32&&j<127)
	return((i<<8)+j+0x8080);
      ungetc(j,fp);
    }
    else if(KANJI==4&&i>32&&i<127){
      j=fgetc(fp);
      if(j>32&&j<127)
	return((i<<8)+j+0x18080);
      ungetc(j,fp);
    }
    else if(KANJI==6&&i>32&&i<96)
      return(kanatbl[i-33]);
    else if((KANJI&1)&&(i<32||(i>='@'&&i<='Z')||(i>='a'&&i<='z')))
      KANJI&=6;
    return(i);
  }
  return(-1);
}

hanzi0(c)
int c;
{
  if(HANZI){
    if(GB==5)
      printf("~}");
    else if((GB&2)==0)
      printf("\033(B");
    HANZI=0;
  }
  putchar(c);
}

hanzi1(c)
int c;
{
  if(HANZI==0){
    switch(GB){
      case 0:
	printf("\033$(E");
	break;
      case 1:
	printf("\033$A");
	break;
      case 5:
	printf("~{");
    }
    HANZI=1;
  }
  if(GB&2)
    printf("%c%c",c>>8,c&255);
  else
    printf("%c%c",(c>>8)&127,c&127);
}

unsigned short nulltbl[]={
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
};
